import urllib
import urllib2
import json

from django.conf import settings

from telegram.handlers.base import BasePlatformHandler


class WebserviceHandler(BasePlatformHandler):

    def handle(self):
        webservice_url = self.subscription.subscriptionmeta_set.get(
                key='webservice_url').value
        payload = json.dumps({
                'channel': self.subscription.channel.name,
                'subject': self.telegram.subject,
                'content': self.telegram.content
                })
        data = urllib.urlencode({'payload': payload})
        request = urllib2.Request(webservice_url, data)
        response = urllib2.urlopen(request)



from telegram.handlers.base import BasePlatformHandler


class DummyHandler(BasePlatformHandler):

    def handle(self):
        print 'Channel: %s' % self.telegram.channel.name
        print 'Subject: %s' % self.telegram.subject
        print 'Message: %s' % self.telegram.content
        print 'Level: %s' % self.subscription.get_level_display()
        print 'Extra: %s' % self.extra



from django.core.management.base import BaseCommand, CommandError

from telegram.models import Platform

from telegram.utils import import_class


class Command(BaseCommand):

    def handle(self, *args, **options):
        if len(args) > 3 or len(args) < 2:
            raise CommandError('Usage: python manage.py createplatform platform_name path.to.Handler ["this is my platform description"]')
        platform_name = args[0]
        platform_handler = args[1]
        try:
            import_class(platform_handler)
        except:
            raise CommandError('Not a valid platform handler or it does not exist')
        try:
            description = args[2]
        except IndexError:
            description = None
        try:
            platform = Platform.objects.get(name=platform_name)
            raise CommandError('Platform %s already exists' % platform_name)
        except Platform.DoesNotExist:
            platform = Platform(name=platform_name, handler=platform_handler, description=description)
            platform.save()
        self.stdout.write('platform %s was successfully created\n' % platform.name)



from django.conf.urls.defaults import patterns, url


urlpatterns = patterns('telegram.views',
        url(r'^channels/$', 'channel_list', name='telegram_channel_list'),
        url(r'^channels/(?P<channel_id>\d+)/$', 'channel_view', name='telegram_channel_view'),
        url(r'^channels/new/$', 'channel_new', name='telegram_channel_new'),
        url(r'^channels/create/$', 'channel_create', name='telegram_channel_create'),
        url(r'^channels/(?P<channel_id>\d+)/edit/$', 'channel_edit', name='telegram_channel_edit'),
        url(r'^channels/(?P<channel_id>\d+)/update/$', 'channel_update', name='telegram_channel_update'),
        url(r'^channels/(?P<channel_id>\d+)/delete/$', 'channel_delete', name='telegram_channel_delete'),
        url(r'^platforms/$', 'platform_list', name='telegram_platform_list'),
        url(r'^platforms/(?P<platform_id>\d+)/$', 'platform_view', name='telegram_platform_view'),
        url(r'^platforms/new/$', 'platform_new', name='telegram_platform_new'),
        url(r'^platforms/create/$', 'platform_create', name='telegram_platform_create'),
        url(r'^platforms/(?P<platform_id>\d+)/edit/$', 'platform_edit', name='telegram_platform_edit'),
        url(r'^platforms/(?P<platform_id>\d+)/update/$', 'platform_update', name='telegram_platform_update'),
        url(r'^platforms/(?P<platform_id>\d+)/delete/$', 'platform_delete', name='telegram_platform_delete'),
        url(r'^subscriptions/$', 'subscription_list', name='telegram_subscription_list'),
        url(r'^subscriptions/(?P<subscription_id>\d+)/$', 'subscription_view', name='telegram_subscription_view'),
        url(r'^subscriptions/new/$', 'subscription_new', name='telegram_subscription_new'),
        url(r'^subscriptions/create/$', 'subscription_create', name='telegram_subscription_create'),
        url(r'^subscriptions/(?P<subscription_id>\d+)/edit/$', 'subscription_edit', name='telegram_subscription_edit'),
        url(r'^subscriptions/(?P<subscription_id>\d+)/update/$', 'subscription_update', name='telegram_subscription_update'),
        url(r'^subscriptions/(?P<subscription_id>\d+)/delete/$', 'subscription_delete', name='telegram_subscription_delete'),
)



from django.core.management.base import BaseCommand, CommandError

from telegram.models import Channel


class Command(BaseCommand):

    def handle(self, *args, **options):
        if len(args) > 2 or len(args) < 1:
            raise CommandError('Usage: python manage.py createchannel channel_name ["this is my channel description"]')
        channel_name = args[0]
        try:
            description = args[1]
        except IndexError:
            description = None
        try:
            channel = Channel.objects.get(name=channel_name)
            raise CommandError('Channel %s already exists' % channel_name)
        except Channel.DoesNotExist:
            channel = Channel(name=channel_name, description=description)
            channel.save()
        self.stdout.write('Channel %s was successfully created\n' % channel.name)



class BaseChannelHandler(object):

    def __init__(self, telegram, channel, level):
        self.telegram = telegram
        self.channel = channel
        self.level = level

    def handle(self):
        raise NotImplementedError('`handle` needs to be defined')


class BasePlatformHandler(object):

    def __init__(self, telegram, subscription, platform, user=None, **kwargs):
        self.telegram = telegram
        self.subscription = subscription
        self.subscription_meta = subscription.subscriptionmeta_set.all()
        self.platform = platform
        self.user = user
        self.extra = kwargs

    def handle(self):
        raise NotImplementedError('`handle` needs to be defined')



from datetime import datetime

from django.conf import settings

from telegram.models import (Telegram, SendLog, Channel, LEVEL_CHOICES,
        Subscription)
from telegram.exceptions import ChannelDoesNotExist, LevelDoesNotExist
from telegram.utils import import_class


FORCE_QUEUEING = getattr(settings, 'TELEGRAM_FORCE_QUEUEING', False)


def send_telegram(channel, subject, message, level='INFO', add_to_queue=True, **kwargs):
    try:
        channel = Channel.objects.get(name=channel)
    except Channel.DoesNotExist:
        raise ChannelDoesNotExist('Channel %s does not exist. Cannot proceed'
                ' with sending the message.' % channel)
    existing_level = None
    for lvl in LEVEL_CHOICES:
        if lvl[1] == level.upper():
            existing_level = lvl[0]
            break
        else:
            continue
    if not existing_level:
        raise LevelDoesNotExist('Level %s does not exist. Cannot proceed'
                ' with sending the message.' % level)
    return _build_telegrams(channel, subject, message, existing_level, add_to_queue=add_to_queue, **kwargs)


def _build_telegrams(channel, subject, message, level, add_to_queue, **kwargs):
    telegram = Telegram(
            subject=subject,
            content=message,
            level=level,
            channel=channel,
            additional_arguments=kwargs)
    telegram.save()
    if channel.handler:
        handler = import_class(channel.handler)(telegram, channel, level)
        subscriptions = handler.handle()
    else:
        subscriptions = Subscription.objects.filter(
                channel=channel, level__gte=level, disabled=False)
    for subscription in subscriptions:
        for sub_plat in subscription.subscriptionplatform_set.all():
            send_log = SendLog(
                    telegram=telegram,
                    subscription_platform=sub_plat
            )
            send_log.save()
            if not FORCE_QUEUEING:
                if not add_to_queue:
                    _send_now(send_log)


def send_all_unsent_telegrams():
    send_logs = SendLog.objects.filter(sent=False)
    for send_log in send_logs:
        _send_now(send_log)


def _send_now(send_log):
    platform = send_log.subscription_platform.platform
    subscription = send_log.subscription_platform.subscription
    telegram = send_log.telegram
    handler = import_class(platform.handler)
    handler = handler(telegram, subscription, platform, **telegram.additional_arguments)
    handler.handle()
    send_log.sent = True
    send_log.sent_at = datetime.now()
    send_log.save()



from django.db import models
from django.contrib.auth.models import User

from jsonfield import JSONField

DEBUG = 1
INFO = 2
SUCCESS = 3
WARNING = 4
ERROR = 5
CRITICAL = 6

LEVEL_CHOICES = (
        (DEBUG, 'DEBUG'),
        (INFO, 'INFO'),
        (SUCCESS, 'SUCCESS'),
        (WARNING, 'WARNING'),
        (ERROR, 'ERROR'),
        (CRITICAL, 'CRITICAL'),
)


class Subscription(models.Model):

    user = models.ForeignKey(User)
    channel = models.ForeignKey('Channel')
    level = models.IntegerField(default=INFO,
            choices=LEVEL_CHOICES)
    platforms = models.ManyToManyField('Platform',
            through='SubscriptionPlatform')
    disabled = models.BooleanField(default=False)

    def __unicode__(self):
        return '%d - %s subscribed to: %s' % (self.pk,
                self.user.username,
                self.channel.name)


class SubscriptionMeta(models.Model):

    key = models.CharField(max_length=255)
    value = models.TextField()
    subscription = models.ForeignKey('Subscription')

    def __unicode__(self):
        return '%s: %s' % (self.key, self.value)


class Channel(models.Model):

    name = models.CharField(max_length=255)
    description = models.TextField(null=True, blank=True)
    handler = models.CharField(max_length=255, null=True, blank=True)

    def __unicode__(self):
        return self.name


class Platform(models.Model):

    name = models.CharField(max_length=255)
    description = models.TextField(null=True, blank=True)
    handler = models.CharField(max_length=255)

    def __unicode__(self):
        return self.name


class PlatformMeta(models.Model):

    key = models.CharField(max_length=255)
    value = models.TextField()
    platform = models.ForeignKey('Platform')

    def __unicode__(self):
        return self.key


class SubscriptionPlatform(models.Model):

    subscription = models.ForeignKey('Subscription')
    platform = models.ForeignKey('Platform')


class Telegram(models.Model):

    channel = models.ForeignKey('Channel')
    subject = models.CharField(max_length=255)
    content = models.TextField()
    level = models.IntegerField(choices=LEVEL_CHOICES)
    additional_arguments = JSONField()
    created_at = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return '%s: %s' % (self.channel, self.subject)

    @property
    def extras(self):
        return self. additional_arguments


class SendLog(models.Model):

    telegram = models.ForeignKey('Telegram')
    subscription_platform = models.ForeignKey('SubscriptionPlatform')
    sent = models.BooleanField(default=False)
    sent_at = models.DateTimeField(null=True, blank=True)



from django.core.mail import send_mail
from django.conf import settings

from telegram.handlers.base import BasePlatformHandler
from telegram.models import PlatformMeta


class EmailHandler(BasePlatformHandler):

    def handle(self):
        """
        Will try to use settings.TELEGRAM_EMAIL_HANDLER_FROM,
        the platformmeta setting "subject_prepend", and the subscriptionmeta
        setting "email_address".
        """
        try:
            meta = self.platform.platformmeta_set.get(key='subject_prepend')
            subject = '%s: %s' % (meta.value, self.telegram.subject)
        except PlatformMeta.DoesNotExist:
            subject = self.telegram.subject
        try:
            from_address = self.platform.platformmeta_set.get(key='from_address').value
        except PlatformMeta.DoesNotExist:
            from_address = settings.TELEGRAM_EMAIL_HANDLER_FROM
        send_mail(
                subject,
                self.telegram.content,
                from_address,
                [self.subscription.subscriptionmeta_set.get(key='email_address').value])




def import_class(name):
    module = name.split('.')[:-1]
    klass = name.split('.')[-1:].pop()
    module = __import__('.'.join(module), fromlist=[klass])
    klass = getattr(module, klass)
    return klass



from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext

from telegram.forms import ChannelForm, PlatformForm, SubscriptionForm
from telegram.models import Channel, Platform, Subscription


def channel_new(request, template_name='telegram/channel_new.html',
        form_class=ChannelForm):
    form = ChannelForm()
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'form': form}, context_instance=context)


def channel_create(request, form_class=ChannelForm):
    if request.method != 'POST':
        return HttpResponseNotFound
    form = ChannelForm(data=request.POST)
    if form.is_valid():
        channel = form.save()
        return HttpResponseRedirect(reverse('telegram_channel_view'))


def channel_view(request, channel_id, template_name='telegram/channel_view.html'):
    channel = get_object_or_404(Channel, pk=channel_id)
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'channel': channel}, context_instance=context)


def channel_list(request, template_name='telegram/channel_list.html'):
    channels = Channel.objects.all()
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'channels':channels}, context_instance=context)


def channel_edit(request, channel_id, template_name='telegram/channel_edit.html',
        form_class=ChannelForm):
    channel = get_object_or_404(Channel, pk=channel_id)
    form = ChannelForm(instance=channel)
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'form': form}, context_instance=context)


def channel_update(request, channel_id, form_class=ChannelForm):
    if request.method != 'POST':
        return HttpResponseNotFound


def channel_delete(request, channel_id):
    channel = get_object_or_404(Channel, pk=channel_id)
    channel.delete()
    return HttpResponseRedirect(reverse('telegram_channel_list'))


def platform_new(request, template_name='telegram/platform_new.html',
        form_class=PlatformForm):
    form = PlatformForm()
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'form': form}, context_instance=context)


def platform_create(request, form_class=PlatformForm):
    if request.method != 'POST':
        return HttpResponseNotFound
    form = PlatformForm(data=request.POST)
    if form.is_valid():
        platform = form.save()
        return HttpResponseRedirect(reverse('telegram_platform_view'))


def platform_view(request, platform_id, template_name='telegram/platform_view.html'):
    platform = get_object_or_404(Platform, pk=platform_id)
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'platform': platform}, context_instance=context)


def platform_list(request, template_name='telegram/platform_list.html'):
    platforms = Platform.objects.all()
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'platforms':platforms}, context_instance=context)


def platform_edit(request, platform_id, template_name='telegram/platform_edit.html',
        form_class=PlatformForm):
    platform = get_object_or_404(Platform, pk=platform_id)
    form = PlatformForm(instance=platform)
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'form': form}, context_instance=context)


def platform_update(request, platform_id, form_class=PlatformForm):
    if request.method != 'POST':
        return HttpResponseNotFound


def platform_delete(request, platform_id):
    platform = get_object_or_404(Platform, pk=platform_id)
    platform.delete()
    return HttpResponseRedirect(reverse('telegram_platform_list'))


def subscription_new(request, template_name='telegram/subscription_new.html',
        form_class=SubscriptionForm):
    form = SubscriptionForm()
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'form': form}, context_instance=context)


def subscription_create(request, form_class=SubscriptionForm):
    if request.method != 'POST':
        return HttpResponseNotFound
    form = SubscriptionForm(data=request.POST)
    if form.is_valid():
        subscription = form.save()
        return HttpResponseRedirect(reverse('telegram_subscription_view'))


def subscription_view(request, subscription_id, template_name='telegram/subscription_view.html'):
    subscription = get_object_or_404(Subscription, pk=subscription_id)
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'subscription': subscription}, context_instance=context)


def subscription_list(request, template_name='telegram/subscription_list.html'):
    subscriptions = Subscription.objects.all()
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'subscriptions':subscriptions}, context_instance=context)


def subscription_edit(request, subscription_id, template_name='telegram/subscription_edit.html',
        form_class=SubscriptionForm):
    subscription = get_object_or_404(Subscription, pk=subscription_id)
    form = SubscriptionForm(instance=subscription)
    context = RequestContext(request)
    return render_to_response(template_name=template_name,
            dictionary={'form': form}, context_instance=context)

def subscription_update(request, subscription_id, form_class=SubscriptionForm):
    if request.method != 'POST':
        return HttpResponseNotFound


def subscription_delete(request, subscription_id):
    subscription = get_object_or_404(Subscription, pk=subscription_id)
    subscription.delete()
    return HttpResponseRedirect(reverse('telegram_subscription_list'))



from django.contrib import admin
from telegram.models import *


class SubscriptionPlatformInline(admin.TabularInline):

    model = SubscriptionPlatform
    extra = 1


class SubscriptionAdmin(admin.ModelAdmin):

    fields = ('channel', 'level', 'disabled',)
    inlines = (SubscriptionPlatformInline,)

    def save_model(self, request, obj, form, change):
        obj.user = request.user
        obj.save()


class PlatformAdmin(admin.ModelAdmin):

    pass


admin.site.register(Telegram)
admin.site.register(Subscription, SubscriptionAdmin)
admin.site.register(Platform, PlatformAdmin)
admin.site.register(SubscriptionMeta)
admin.site.register(Channel)
admin.site.register(SubscriptionPlatform)
admin.site.register(SendLog)



from django.forms import ModelForm
from telegram.models import Channel, Platform, Subscription


class ChannelForm(ModelForm):

    class Meta:
        model = Channel


class PlatformForm(ModelForm):

    class Meta:
        model = Platform


class SubscriptionForm(ModelForm):

    class Meta:
        model = Subscription



from setuptools import setup, find_packages
import sys, os

version = '0.1.0'

setup(name='django-telegram',
      version=version,
      description="A messaging framework that supports email, irc, notify my android and others",
      long_description="""\
""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='messaging, push, django',
      author='Kyle Terry',
      author_email='kyle@kyleterry.com',
      url='https://github.com/aquameta/django-telegram',
      license='',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
          'django-jsonfield',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
